<?php

namespace Drupal\metatag_ui_enhancements\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the configuration for metatags default images.
 */
class MetatagUiEnhancementsSettingsForm extends FormBase {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The image style storage.
   *
   * @var \Drupal\image\ImageStyleStorageInterface
   */
  protected $imageStyleStorage;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The metatag_ui_enhancements service.
   *
   * @var \Drupal\metatag_ui_enhancements\Service\MetatagUiEnhancementsService
   */
  protected $metatagUiEnhancementsService;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->moduleHandler = $container->get('module_handler');
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->imageStyleStorage = $container->get('entity_type.manager')->getStorage('image_style');
    $instance->state = $container->get('state');
    $instance->metatagUiEnhancementsService = $container->get('metatag_ui_enhancements.service');
    $instance->configFactory = $container->get('config.factory');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'metatag_enhancements_ui_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $image_styles = $this->imageStyleStorage->loadMultiple();

    $options = [];
    foreach ($image_styles as $image_style) {
      $options[$image_style->id()] = $image_style->label();
    }

    $defaultValues = $this->metatagUiEnhancementsService->getImageFieldValues();

    foreach ($this->metatagUiEnhancementsService->getImageFields() as $key => $imageField) {
      $form[$key] = [
        '#type' => 'details',
        '#title' => $this->t('%type settings', ['%type' => $imageField['title']]),
        '#open' => FALSE,
        '#tree' => TRUE,
      ];

      foreach ($imageField['fields'] as $field_key => $field_label) {
        $image_id_token = "[metatag-default-image:$key]";
        $style_token = "[metatag-default-image:$key:style]";

        $form[$key][$field_key]['id'] = [
          '#type' => 'media_library',
          '#title' => $this->t('%type - Select an image', ['%type' => $field_label]),
          '#allowed_bundles' => ['image'],
          '#default_value' => $defaultValues[$key]['id'] ?? NULL,
          '#description' => $this->t('Choose an image from the media library.'),
        ];

        $form[$key][$field_key]['style'] = [
          '#type' => 'select',
          '#title' => $this->t('%type - Select a style', ['%type' => $field_label]),
          '#default_value' => $defaultValues[$key]['style'] ?? NULL,
          '#description' => $this->t('Choose a style for the image.'),
          '#options' => $options,
        ];

        $form[$key][$field_key]['token'] = [
          '#type' => 'markup',
          '#markup' => $this->t('Use these tokens: @image_id_token, @style_token', [
            '@image_id_token' => $image_id_token,
            '@style_token' => $style_token,
          ]),
        ];
      }
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Get Global metatags config to update when default images are configured.
    $config = $this->configFactory->getEditable('metatag.metatag_defaults.global');
    $tags = $config->get('tags');

    $metatag_ui_enhancements = [];
    foreach ($this->metatagUiEnhancementsService->getImageFields() as $key => $imageField) {
      foreach ($imageField['fields'] as $field_key => $field_label) {
        $value = $form_state->getValue([$key, $field_key, 'id']);
        if (!$value) {
          continue;
        }
        $metatag_ui_enhancements[$key][$field_key]['id'] = $value;

        $style = $form_state->getValue([$key, $field_key, 'style']);
        $metatag_ui_enhancements[$key][$field_key]['style'] = $style;
        $metatag_ui_enhancements[$key][$field_key]['url'] = $this->metatagUiEnhancementsService->getImageUrlWithStyle($value, $style);
        $tags[$field_key] = "[metatag-default-image:$key:$style]";
      }
    }
    // Store default images metatags in state.
    $this->state->set('metatag_ui_enhancements', $metatag_ui_enhancements);

    // Update global metatags default images with the ones set here.
    $config->set('tags', $tags);
    $config->save();

    $this->messenger()->addMessage('Default images for metatags has been saved');

  }

}
