<?php

declare(strict_types=1);

namespace Drupal\metatag_ui_enhancements\Service;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\file\FileInterface;

/**
 * MetatagUiEnhancement Service class.
 */
class MetatagUiEnhancementsService {

  use StringTranslationTrait;

  /**
   * Constructs a new MetatagUiEnhancementsTokenService.
   *
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity repository.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager.
   */
  public function __construct(
    protected StateInterface $state,
    protected ModuleHandlerInterface $moduleHandler,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected EntityFieldManagerInterface $entityFieldManager,
  ) {}

  /**
   * Get image fields.
   *
   * @return array
   *   An array of image fields.
   */
  public function getImageFields(): array {
    $imageFields = [
      'advanced' => [
        'title' => 'Advanced',
        'fields' => [
          'image_src' => 'Advanced image',
        ],
      ],
    ];

    if ($this->moduleHandler->moduleExists('metatag_open_graph')) {
      $imageFields['open_graph'] = [
        'title' => 'OpenGraph',
        'fields' => [
          'og_image' => 'Open Graph image',
        ],
      ];
    }

    if ($this->moduleHandler->moduleExists('metatag_twitter_cards')) {
      $imageFields['twitter_cards'] = [
        'title' => 'X (Twitter) Cards',
        'fields' => [
          'twitter_cards_image' => 'X (Twitter) image',
        ],
      ];
    }

    return $imageFields;
  }

  /**
   * Get image field values from state.
   *
   * @return array
   *   An array of image field values.
   */
  public function getImageFieldValues() {
    $state = $this->state->get('metatag_ui_enhancements', []);
    $values = [];
    foreach ($this->getImageFields() as $key => $imageField) {
      foreach ($imageField['fields'] as $field_key => $field_label) {
        $values[$key] = [
          'id' => $state[$key][$field_key]['id'] ?? NULL,
          'url' => $state[$key][$field_key]['url'] ?? NULL,
          'style' => $state[$key][$field_key]['style'] ?? NULL,
        ];
      }
    }
    return $values;
  }

  /**
   * Gets a styled image URL.
   */
  public function getImageUrlWithStyle($media_id, $style_name = NULL) {
    if (!$media_id) {
      return NULL;
    }
    $media = $this->entityTypeManager->getStorage('media')->load($media_id);
    if (!$media) {
      return NULL;
    }
    if ($media->bundle() !== 'image') {
      return NULL;
    }
    // Iterate over fields to find the image field.
    $image_field_name = NULL;
    foreach ($media->getFieldDefinitions() as $field_name => $field_definition) {
      if ($field_definition->getType() === 'image') {
        $image_field_name = $field_name;
      }
    }

    if (!$image_field_name) {
      return NULL;
    }

    $field_media_image = $media->get($image_field_name)->entity;

    if (!$field_media_image instanceof FileInterface) {
      return NULL;
    }

    $image_style = $this->entityTypeManager->getStorage('image_style')->load($style_name);
    if ($image_style) {
      return $image_style->buildUrl($field_media_image->getFileUri());
    }

    return $field_media_image->createFileUrl();
  }

  /**
   * Get metadata field from entity.
   */
  public function getMetadataFields($entity) {
    $fields = [];

    if ($entity->getEntityTypeId() !== 'field_config') {
      $field_definitions = $this->entityFieldManager->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle());
      foreach ($field_definitions as $field_name => $field_definition) {
        if ($field_definition->getType() === 'metatag') {
          $fields[] = $field_name;
        }
      }
    }

    return reset($fields);
  }

}
